@@ -5,7 +5,7 @@ package Dancer2::Session::PSGI;
 use Moo;
 with 'Dancer2::Core::Role::SessionFactory';
 
-our $VERSION = '0.008'; # VERSION
+our $VERSION = '0.009'; # VERSION
 
 #-----------------------------------------#
 # Alter SessionFactory attribute defaults
@@ -28,6 +28,10 @@ sub _flush {
     $self->request->env->{'psgix.session'} = $data;
 }
 
+# Do nothing - we override change_id below but this method must exist to
+# force Dance2::Core::App::change_session_id to call our change_id method
+sub _change_id {}
+
 # Middleware handles cookie expiry
 sub _destroy { return }
 
@@ -40,6 +44,16 @@ sub _sessions { return [] }
 # Overridden methods from SessionFactory
 #-----------------------------------------#
 
+sub change_id {
+    my ( $self, %params ) = @_;
+    my $session = $params{session};
+
+    $self->execute_hook( 'engine.session.before_change_id', $session->id );
+    $self->request->env->{'psgix.session.options'}->{'change_id'} = 1;
+    # we don't know what the new session ID will be yet so make it undef
+    $self->execute_hook( 'engine.session.before_change_id', undef );
+}
+
 # Middleware sets the cookie.
 sub set_cookie_header {
     my ( $self, %params ) = @_;
@@ -71,7 +85,7 @@ Dancer2::Session::PSGI - Dancer2 session storage via Plack::Middleware::Session
 
 =head1 VERSION
 
-version 0.008
+version 0.009
 
 =head1 SYNOPSIS
 
@@ -92,6 +106,7 @@ L<Plack::Middleware::Session> for session management.
 
 =for Pod::Coverage method_names_here
 set_cookie_header
+change_id
 
 =head1 CONFIGURATION
 
